;SlurUp
(d-DirectivePut-standalone-display "SlurDirection" "SlurUp")
(d-MoveCursorLeft)
(d-DirectivePut-standalone-postfix "SlurDirection" " \\slurUp" )
(d-DirectivePut-standalone-minpixels "SlurDirection" 40)
(d-MoveCursorRight)
(d-RefreshDisplay) 