;;;; NewStructuredStaff
(let ()
(d-MoveToBeginning)	
(d-AddAfter)
(d-MoveToStaffUp)
(while (d-MoveCursorRight) (disp "Measure Number " (d-GetMeasure) "\n")
	(if (Timesignature?)
		(begin
			(d-PushPosition)
			(d-UnsetMark)
			(d-SetMark)
			(d-Copy)
			(d-MoveToStaffDown)
			(d-Paste) 
			(d-PopPosition))))
(d-MoveToStaffDown)
(d-MoveToBeginning))
