(d-DeactivateTimeSignatureStencil)
(d-TransposeOctaveUp)
(d-HideEmptyStaffs)
(d-RaggedBottom)
(d-BookComposer "composer")
(d-BookInstrumentation "Full Score")
(d-BookCopyright "Copyright © Denemo 2014")
(d-BookTitle "My Title")
(d-NewMovement)
;(d-DeleteMovement)
(d-HideBarlines)
;(d-TitledPiece "My Movement Title")
(d-Custos)
(d-MovementPageBreak)
;(d-NewStaffAfter)
(d-SharpenInitialKeysigs)
(d-InitialTimeSig "6/8")
(d-AddDuplicateMeasure)
(d-Set3)
(d-D)
(d-ToggleBeginSlur)
(d-InsertE)
(d-ToggleEndSlur)
(d-BreathMark)
(d-InsertF)
